@@ -1,8 +1,9 @@
 import logging
 from argparse import ArgumentParser, Namespace
+from pathlib import Path
 from typing import Any, Callable
 
-from wkcuber.api.Dataset import WKDataset
+from wkcuber.api.dataset import Dataset
 from wkcuber.api.bounding_box import BoundingBox, BoundingBoxNamedTuple
 import numpy as np
 
@@ -36,10 +37,12 @@ def named_partial(func: Callable, *args: Any, **kwargs: Any) -> Callable:
 def create_parser() -> ArgumentParser:
     parser = ArgumentParser()
 
-    parser.add_argument("source_path", help="Path to input WKW dataset")
+    parser.add_argument("source_path", help="Path to input WKW dataset", type=Path)
 
     parser.add_argument(
-        "target_path", help="WKW dataset with which to compare the input dataset."
+        "target_path",
+        help="WKW dataset with which to compare the input dataset.",
+        type=Path,
     )
 
     parser.add_argument(
@@ -56,31 +59,33 @@ def create_parser() -> ArgumentParser:
 
 
 def assert_equality_for_chunk(
-    source_path: str,
-    target_path: str,
+    source_path: Path,
+    target_path: Path,
     layer_name: str,
     mag: Mag,
     sub_box: BoundingBoxNamedTuple,
 ) -> None:
-    wk_dataset = WKDataset(source_path)
-    layer = wk_dataset.layers[layer_name]
+    wk_dataset = Dataset(source_path)
+    layer = wk_dataset.get_layer(layer_name)
     backup_wkw_info = WkwDatasetInfo(target_path, layer_name, mag, header=None)
     with open_wkw(backup_wkw_info) as backup_wkw:
-        mag_ds = layer.get_mag(mag)
+        mag_view = layer.get_mag(mag)
         logging.info(f"Checking sub_box: {sub_box}")
 
-        data = mag_ds.read(sub_box.topleft, sub_box.size)
+        data = mag_view.read(sub_box.topleft, sub_box.size)
         backup_data = backup_wkw.read(sub_box.topleft, sub_box.size)
         assert np.all(
             data == backup_data
         ), f"Data differs in bounding box {sub_box} for layer {layer_name} with mag {mag}"
 
 
-def check_equality(source_path: str, target_path: str, args: Namespace = None) -> None:
+def check_equality(
+    source_path: Path, target_path: Path, args: Namespace = None
+) -> None:
 
     logging.info(f"Comparing {source_path} with {target_path}")
 
-    wk_src_dataset = WKDataset(source_path)
+    wk_src_dataset = Dataset(source_path)
     src_layer_names = list(wk_src_dataset.layers.keys())
     target_layer_names = [
         layer["name"] for layer in detect_layers(target_path, 0, False)
