@@ -8,7 +8,7 @@ from typing import Tuple, Optional, Union, cast
 import nibabel as nib
 import numpy as np
 
-from wkcuber.api.Dataset import TiffDataset, WKDataset
+from wkcuber.api.dataset import Dataset
 from wkcuber.api.bounding_box import BoundingBox
 from wkcuber.utils import (
     DEFAULT_WKW_FILE_LEN,
@@ -28,10 +28,11 @@ def create_parser() -> ArgumentParser:
         "source_path",
         help="Path to NIFTY file or to a directory if multiple NIFTI files should be converted. "
         "In the latter case, also see --color_file and --segmentation_file.",
+        type=Path,
     )
 
     parser.add_argument(
-        "target_path", help="Output directory for the generated WKW dataset."
+        "target_path", help="Output directory for the generated WKW dataset.", type=Path
     )
 
     parser.add_argument(
@@ -66,13 +67,6 @@ def create_parser() -> ArgumentParser:
         default=None,
     )
 
-    parser.add_argument(
-        "--write_tiff",
-        help="Output tiff dataset instead of wkw.",
-        default=False,
-        action="store_true",
-    )
-
     parser.add_argument(
         "--use_orientation_header",
         help="Use orientation information from header to interpret the input data (should be tried if output orientation seems to be wrong).",
@@ -134,7 +128,6 @@ def convert_nifti(
     is_segmentation_layer: bool = False,
     file_len: int = DEFAULT_WKW_FILE_LEN,
     bbox_to_enforce: BoundingBox = None,
-    write_tiff: bool = False,
     use_orientation_header: bool = False,
     flip_axes: Optional[Union[int, Tuple[int, ...]]] = None,
 ) -> None:
@@ -210,31 +203,17 @@ def convert_nifti(
         ),
     )
 
-    if write_tiff:
-        tiff_ds = TiffDataset.get_or_create(
-            target_path, scale=cast(Tuple[float, float, float], scale or (1, 1, 1))
-        )
-        tiff_layer = tiff_ds.get_or_add_layer(
-            layer_name,
-            category_type,
-            dtype_per_layer=np.dtype(dtype),
-            **max_cell_id_args,
-        )
-        tiff_mag = tiff_layer.get_or_add_mag("1")
-
-        tiff_mag.write(cube_data.squeeze())
-    else:
-        wk_ds = WKDataset.get_or_create(
-            target_path, scale=cast(Tuple[float, float, float], scale or (1, 1, 1))
-        )
-        wk_layer = wk_ds.get_or_add_layer(
-            layer_name,
-            category_type,
-            dtype_per_layer=np.dtype(dtype),
-            **max_cell_id_args,
-        )
-        wk_mag = wk_layer.get_or_add_mag("1", file_len=file_len)
-        wk_mag.write(cube_data)
+    wk_ds = Dataset.get_or_create(
+        target_path, scale=cast(Tuple[float, float, float], scale or (1, 1, 1))
+    )
+    wk_layer = wk_ds.get_or_add_layer(
+        layer_name,
+        category_type,
+        dtype_per_layer=np.dtype(dtype),
+        **max_cell_id_args,
+    )
+    wk_mag = wk_layer.get_or_add_mag("1", file_len=file_len)
+    wk_mag.write(cube_data)
 
     logging.debug(
         "Converting of {} took {:.8f}s".format(
@@ -251,7 +230,6 @@ def convert_folder_nifti(
     scale: Tuple[float, ...],
     use_orientation_header: bool = False,
     bbox_to_enforce: BoundingBox = None,
-    write_tiff: bool = False,
     flip_axes: Optional[Union[int, Tuple[int, ...]]] = None,
 ) -> None:
     paths = list(source_folder_path.rglob("**/*.nii"))
@@ -287,7 +265,6 @@ def convert_folder_nifti(
                 "uint8",
                 scale,
                 is_segmentation_layer=False,
-                write_tiff=write_tiff,
                 bbox_to_enforce=bbox_to_enforce,
                 use_orientation_header=use_orientation_header,
                 flip_axes=flip_axes,
@@ -300,7 +277,6 @@ def convert_folder_nifti(
                 "uint8",
                 scale,
                 is_segmentation_layer=True,
-                write_tiff=write_tiff,
                 bbox_to_enforce=bbox_to_enforce,
                 use_orientation_header=use_orientation_header,
                 flip_axes=flip_axes,
@@ -313,7 +289,6 @@ def convert_folder_nifti(
                 "uint8",
                 scale,
                 is_segmentation_layer=False,
-                write_tiff=write_tiff,
                 bbox_to_enforce=bbox_to_enforce,
                 use_orientation_header=use_orientation_header,
                 flip_axes=flip_axes,
@@ -321,7 +296,7 @@ def convert_folder_nifti(
 
 
 def main(args: Namespace) -> None:
-    source_path = Path(args.source_path)
+    source_path = args.source_path
 
     flip_axes = None
     if args.flip_axes is not None:
@@ -333,7 +308,6 @@ def main(args: Namespace) -> None:
 
     conversion_args = {
         "scale": args.scale,
-        "write_tiff": args.write_tiff,
         "bbox_to_enforce": args.enforce_bounding_box,
         "use_orientation_header": args.use_orientation_header,
         "flip_axes": flip_axes,
@@ -342,7 +316,7 @@ def main(args: Namespace) -> None:
     if source_path.is_dir():
         convert_folder_nifti(
             source_path,
-            Path(args.target_path),
+            args.target_path,
             args.color_file,
             args.segmentation_file,
             **conversion_args,
@@ -350,7 +324,7 @@ def main(args: Namespace) -> None:
     else:
         convert_nifti(
             source_path,
-            Path(args.target_path),
+            args.target_path,
             args.layer_name,
             args.dtype,
             is_segmentation_layer=args.is_segmentation_layer,
