@@ -2,6 +2,7 @@ import functools
 import re
 import time
 from concurrent.futures._base import Future
+from pathlib import Path
 from types import TracebackType
 
 import logging
@@ -48,14 +49,14 @@ FallbackArgs = namedtuple("FallbackArgs", ("distribution_strategy", "jobs"))
 BLOCK_LEN = 32
 DEFAULT_WKW_FILE_LEN = 32
 DEFAULT_WKW_VOXELS_PER_BLOCK = 32
-CUBE_REGEX = re.compile(r"z(\d+)/y(\d+)/x(\d+)(\.wkw)$")
+CUBE_REGEX = re.compile(fr"z(\d+){os.path.sep}y(\d+){os.path.sep}x(\d+)(\.wkw)$")
 
 logger = getLogger(__name__)
 
 
 def open_wkw(info: WkwDatasetInfo) -> wkw.Dataset:
     ds = wkw.Dataset.open(
-        path.join(info.dataset_path, info.layer_name, str(info.mag)), info.header
+        str(info.dataset_path / info.layer_name / str(info.mag)), info.header
     )
     return ds
 
@@ -124,7 +125,7 @@ def add_isotropic_flag(parser: argparse.ArgumentParser) -> None:
         help="Activates isotropic downsampling. The default is anisotropic downsampling. "
         "Isotropic downsampling will always downsample each dimension with the factor 2.",
         dest="isotropic",
-        default=False,
+        default=None,
         action="store_true",
     )
 
@@ -138,6 +139,17 @@ def add_interpolation_flag(parser: argparse.ArgumentParser) -> None:
     )
 
 
+def add_sampling_mode_flag(parser: argparse.ArgumentParser) -> None:
+    parser.add_argument(
+        "--sampling_mode",
+        help="There are three different types: "
+        "'auto' - The next magnification is chosen so that the width, height and depth of a downsampled voxel assimilate. For example, if the z resolution is worse than the x/y resolution, z won't be downsampled in the first downsampling step(s). As a basis for this method, the scale from the datasource-properties.json is used. "
+        "'isotropic' - Each dimension is downsampled equally. "
+        "'constant_z' - The x and y dimensions are downsampled equally, but the z dimension remains the same.",
+        default="auto",
+    )
+
+
 def setup_logging(args: argparse.Namespace) -> None:
     logging.basicConfig(
         level=(logging.DEBUG if args.verbose else logging.INFO),
@@ -270,7 +282,7 @@ def wait_and_ensure_success(futures: List[Future]) -> None:
 class BufferedSliceWriter(object):
     def __init__(
         self,
-        dataset_path: str,
+        dataset_path: Path,
         layer_name: str,
         dtype: np.dtype,
         origin: Union[Tuple[int, int, int], List[int]],
@@ -494,3 +506,16 @@ def convert_mag1_offset(
     mag1_offset: Union[List, np.ndarray], target_mag: Mag
 ) -> np.ndarray:
     return np.array(mag1_offset) // target_mag.as_np()  # floor div
+
+
+def get_executor_args(global_args: argparse.Namespace) -> argparse.Namespace:
+    executor_args = argparse.Namespace()
+    executor_args.jobs = global_args.jobs
+    executor_args.distribution_strategy = global_args.distribution_strategy
+    executor_args.job_resources = global_args.job_resources
+    return executor_args
+
+
+def _snake_to_camel_case(snake_case_name: str) -> str:
+    parts = snake_case_name.split("_")
+    return parts[0] + "".join(part.title() for part in parts[1:])
